<?php

/**
 * @file
 * Include functions for the wow_realm module.
 */

/**
 * Menu callback; Retrieve a JSON object containing autocomplete suggestion for
 * existing realms.
 */
function wow_realm_autocomplete($string = '') {
  $matches = array();
  if ($string) {
    $result = db_select('wow_realm')
      ->fields('wow_realm', array('slug', 'region', 'name'))
      ->condition('name', db_like($string) . '%', 'LIKE')
      ->range(0, 10)
      ->execute();
    foreach ($result as $realm) {
      $matches[check_plain($realm->slug)] = '[' . strtoupper($realm->region) . '] ' . check_plain($realm->name);
    }
  }

  drupal_json_output($matches);
}

/**
 * Updates the full list of realms.
 *
 * Fetch all existing realms for a given region, update them in the database.
 *
 * @param string $region
 *  The region API.
 */
function wow_realm_update_process($region, &$context) {

  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['current'] = 0;
    $context['sandbox']['total'] = count(wow_api_info());
  }

  $context['sandbox']['current']++;

  $realms = array();
  // Fetch the realm list for this region and build a realm lookup array.
  foreach (wow_realm_fetch($region)->realms as $realm) {
    $realm->region = $region;
    $realms[check_plain($realm->slug)] = $realm;
  }

  // Load every known realms from the local database.
  $local_realms = db_select('wow_realm')
    ->fields('wow_realm')
    ->condition('slug', array_keys($realms), 'IN')
    ->execute()
    ->fetchAllAssoc('slug');

  // Process the realm list from the service.
  foreach ($realms as $key => $realm) {

    if (array_key_exists($key, $local_realms)) {
      // The realm already exists in the local database. For efficiency
      // manually save the original realm before applying any changes.
      $realm->original = $local_realms[$key];
      $realm->rid = $local_realms[$key]->rid;

      // Remove realm from the array if they are found. Remaining array
      // contains the list of realms who are not existing anymore.
      unset($local_realms[$key]);
    }

    // Save new or existing realms into database.
    wow_realm_save($realm);
  }
  if (!empty($local_realms)) {
    // Process the realms from the remaining array.
    foreach ($local_realms as $realm) {
      $rids[] = $realm->rid;
    }

    wow_realm_delete_multiple($rids);
  }

  $args = array(
    '!total' => count($realms),
    '%region' => wow_api_info($region, 'name'),
  );
  $context['message'] = t('Imported a total of !total realms from %region region.', $args);
}
